package com.sample;

import java.io.InputStreamReader;
import java.io.Reader;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;

/**
 * This is a sample file to launch a rule package from a rule source file.
 */
public class DroolsTest {

    public static final void main(String[] args) {
        try {
        	
        	//load up the rulebase
            RuleBase ruleBase = readRule();
            WorkingMemory workingMemory = ruleBase.newStatefulSession();
            
            //go !
            Message message = new Message();
            message.setStatus(100);
            Wrapper wrapper = new Wrapper();
            wrapper.setMessage(message);
            workingMemory.insert( message );
            workingMemory.insert(wrapper);
            
            message = new Message();
            message.setStatus(100);
            wrapper = new Wrapper();
            wrapper.setMessage(message);
            workingMemory.insert( message );
            workingMemory.insert(wrapper);
            
            message = new Message();
            message.setStatus(101);
            wrapper = new Wrapper();
            wrapper.setMessage(message);
            workingMemory.insert( message );
            workingMemory.insert(wrapper);
            
            message = new Message();
            message.setStatus(102);
            wrapper = new Wrapper();
            wrapper.setMessage(message);
            workingMemory.insert( message );
            workingMemory.insert(wrapper);
            
            
            workingMemory.fireAllRules();   
            
            
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    /**
     * Please note that this is the "low level" rule assembly API.
     */
	private static RuleBase readRule() throws Exception {
		//read in the source
		Reader source = new InputStreamReader( DroolsTest.class.getResourceAsStream( "/Sample.drl" ) );
		
		//optionally read in the DSL (if you are using it).
		//Reader dsl = new InputStreamReader( DroolsTest.class.getResourceAsStream( "/mylang.dsl" ) );

		//Use package builder to build up a rule package.
		//An alternative lower level class called "DrlParser" can also be used...
		
		PackageBuilder builder = new PackageBuilder();

		//this wil parse and compile in one step
		//NOTE: There are 2 methods here, the one argument one is for normal DRL.
		builder.addPackageFromDrl( source );

		//Use the following instead of above if you are using a DSL:
		//builder.addPackageFromDrl( source, dsl );
		
		//get the compiled package (which is serializable)
		Package pkg = builder.getPackage();
		
		//add the package to a rulebase (deploy the rule package).
		RuleBase ruleBase = RuleBaseFactory.newRuleBase();
		ruleBase.addPackage( pkg );
		return ruleBase;
	}
	
	public static class Message {
		private int status;
		
		public int getStatus() {
			return this.status;
		}
		
		public void setStatus( int status ) {
			this.status = status;
		}
	}
	
	public static class Wrapper
	{
		private Message message;
		
		public Message getMessage()
		{
			return this.message;
		}
		
		public void setMessage(Message message)
		{
			this.message = message;
		}
	}
    
}
